import { Constructor, Container, DependencyRegistrator } from 'cheap-di';
import { FC, ReactNode } from 'react';
type Dependency = (dependencyRegistrator: DependencyRegistrator) => void;
type SelfDependency = Constructor;
interface Props {
    /**
     * using of cheap-di API to register dependencies as you wish
     * @example
     * <DIProvider dependencies={[
     *   (dr) => dr.registerImplementation(Foo).as(Bar),
     *   (dr) => dr.registerImplementation(Foo2).as(Bar2),
     *   // ...
     * ]}>
     *   <MyComponent />
     * </DIProvider>
     * */
    dependencies?: Dependency[];
    /**
     * shortcut for container.registerImplementation(<you-class>), when you want to keep code simple, without using interfaces (abstract classes)
     * @example
     * <DIProvider self={[Foo, Bar]}>
     *   <MyComponent />
     * </DIProvider>
     * */
    self?: SelfDependency[];
    /**
     * shortcut for container.registerImplementation(<you-class>).asSingleton()
     * @example
     * <DIProvider selfSingletons={[Foo, Bar]}>
     *   <MyComponent />
     * </DIProvider>
     * */
    selfSingletons?: SelfDependency[];
    /**
     * adds your configured container as root container to your React tree
     * @example
     * import { container } from 'cheap-di';
     *
     * class Foo {}
     *
     * const App = () => {
     *   const [configuredContainer] = useState(() => {
     *     container.registerImplementation(Foo);
     *     // ...
     *   });
     *
     *   return (
     *     <DIProvider parentContainer={configuredContainer}>
     *       <MyComponent />
     *     </DIProvider>
     *   );
     * }
     * */
    parentContainer?: Container;
    /** enables logging of dependencies registrations */
    debugName?: string;
    children: ReactNode;
}
declare const DIProvider: FC<Props>;
export { DIProvider };
export type { Props as DIProviderProps };
